extends CharacterBody2D

class PlayerInputs:
	var motion = Vector2()
	var bombing = false

	func update():
		motion = Vector2()
		if Input.is_action_pressed("move_left"):
			motion += Vector2(-1, 0)
		if Input.is_action_pressed("move_right"):
			motion += Vector2(1, 0)
		if Input.is_action_pressed("move_up"):
			motion += Vector2(0, -1)
		if Input.is_action_pressed("move_down"):
			motion += Vector2(0, 1)

		bombing = Input.is_action_pressed("set_bomb")


const MOTION_SPEED = 90.0
const BOMB_RATE = 0.5

@export
var synced_position = Vector2()

@export
var stunned = false

var inputs = PlayerInputs.new()
var last_bomb_time = BOMB_RATE
var current_anim = ""

func _ready():
	stunned = false
	position = synced_position


func _physics_process(delta):
	if multiplayer.multiplayer_peer == null or str(multiplayer.get_unique_id()) == str(name):
		# The client which this player represent will update the controls state, and notify it to everyone.
		inputs.update()
		if multiplayer.multiplayer_peer != null:
			rpc(&"update_controls", inputs.motion, inputs.bombing)

	if multiplayer.multiplayer_peer == null or is_multiplayer_authority():
		# The server updates the position that will be notified to the clients.
		synced_position = position
		# And increase the bomb cooldown spawning one if the client wants to.
		last_bomb_time += delta
		if not stunned and is_multiplayer_authority() and inputs.bombing and last_bomb_time >= BOMB_RATE:
			last_bomb_time = 0.0
			get_node("../../BombSpawner").spawn([position, str(name).to_int()])
	else:
		# The client simply updates the position to the last known one.
		position = synced_position

	if not stunned:
		# Everybody runs physics. I.e. clients tries to predict where they will be during the next frame.
		motion_velocity = inputs.motion * MOTION_SPEED
		move_and_slide()

	# Also update the animation based on the last known player input state
	var new_anim = "standing"

	if inputs.motion.y < 0:
		new_anim = "walk_up"
	elif inputs.motion.y > 0:
		new_anim = "walk_down"
	elif inputs.motion.x < 0:
		new_anim = "walk_left"
	elif inputs.motion.x > 0:
		new_anim = "walk_right"

	if stunned:
		new_anim = "stunned"

	if new_anim != current_anim:
		current_anim = new_anim
		get_node("anim").play(current_anim)


func set_player_name(value):
	get_node("label").set_text(value)


# Used to update the player input state. Unreliable but ordered
@rpc(any_peer, unreliable_ordered)
func update_controls(motion : Vector2, bombing : bool):
	if str(multiplayer.get_remote_sender_id()) != str(name):
		print_debug("Someones trying to cheat: %d" % multiplayer.get_remote_sender_id())
		return
	inputs.motion = motion
	inputs.bombing = bombing


@rpc(call_local)
func exploded(_by_who):
	if stunned:
		return
	stunned = true
	get_node("anim").play("stunned")
